% Name:         lifetime_prediction.m
% Author:       T.H. Hoksbergen (t.h.hoksbergen@utwente.nl)
% Organisation: University of Twente
% Date:         13-06-2023

%% Close/clear all
close all
clear variables
clc

%% Variables
SI = 10e6;      % Fatigue endurance limit [Pa]
N_SI = 1e7;     % # of cycles at endurance limit [-]
UTS = 67.3e6;   % Ultimate tensile strength [Pa]
n = 5000;       % Number of distributed impact points [-]

%% Fatigue strength functions
m = -log(N_SI)/(log(SI)-log(UTS));
Ni = @(S) (S/UTS).^(-m);        % Fatigue lifetime at stress level S [-]

%% Load stress data
% The stress data is based on a triangular mesh where the variable tri
% contains the triangles, p the node coordinates (r,z), time the time and 
% VM the von Mises stress for each time in each node
load("stress_data.mat");        % 2 mm droplet, 100 m/s on bulk epoxy

%% Single impact point calculations
N_nodes = size(VM,2);           % Number of nodes
D_single = zeros(1,N_nodes);    % Pre initialize damage parameter vatiable

% Loop over all nodes
for i=1:N_nodes
    c = rainflow(VM(:,i));      % Rainflow counting algorithm
    idx = c(:,2)>=SI;           % Find stresses above endurance limit
    N_RF = c(idx,1);            % Store stress cycles above endurance limit
    S_RF = c(idx,2);            % Store stress levels above endurance limit
    % Loop over the number of distinct stress levels for this node
    for j=1:length(N_RF)
        D_single(i) = D_single(i)+N_RF(j)/Ni(S_RF(j)); % Calculate the damage parameter for node i
    end
end

% Plot damage parameter
figure;
trisurf(tri,1e3*p(1,:),1e3*p(2,:),[],D_single);
shading interp; view([0,90]); axis equal tight; colorbar; 
xlim([0,2]); ylim([-1,0]); 
title("Damage parameter for single point impacts"); xlabel("r [mm]"); ylabel("z [mm]");

% Calculate corresponding lifetime
lifetime_single_point = 1/max(D_single);

%% DISTRIBUTED IMPACT LOCATION CALCULATIONS
r_max = max(p(1,:));            % Max radius to consider
r_i = sqrt(rand(1,n))*r_max;    % Set of distributed impact points
N_t = length(time);             % Number of points in time
z2 = [0:-10e-6:-1e-3, -1.1e-3:-100e-6:min(p(2,:))]; % Definition of points in z-direction
N_z = length(z2);               % Number of points in z-direction
S = zeros(N_z,N_t*n);           % Pre initialize interpolated stress variable

% Interpolate stresses
disp("Interpolating stresses, this might take some time.");
tic;
[r,z] = meshgrid(r_i,z2);                           % Define the grid for the results
TR = triangulation(double(tri),p(1,:)',p(2,:)');    % Triangulate the triangles
[ti,bc] = pointLocation(TR,[r(:),z(:)]);            % Find the points of interest
for i=1:N_t
    Vq = VM(i,:)';                                  % Stress at time i
    triVals = Vq(TR(ti,:));                         % Stress values at surrounding nodes
    Vq = dot(bc',triVals')';                        % Interpolated stress
    S(:,((1:n)-1)*N_t+i) = reshape(Vq,N_z,[]);      % Store the interpolated stress value
end
toc;
disp("Done with the interpolation.");

% Rainflow counting
disp("Performing Rainflow counting, this might take some time.");
tic;
D = zeros(1,N_z);
for i=1:N_z
    c = rainflow(S(i,:));       % Perform Rainflow counting at depth i
    idx = c(:,2)>=SI;           % Find stresses above endurance limit
    N_RF = c(idx,1);            % Store stress cycles above endurance limit
    S_RF = c(idx,2);            % Store stress levels above endurance limit
    % Loop over the number of distinct stress levels for this z-coordinate
    for j=1:length(N_RF)
        D(i) = D(i)+N_RF(j)/Ni(S_RF(j)); % Calculate the damage parameter for current z-coordinate
    end
end
toc;
disp("Done with Rainflow counting.");

% Plot damage parameter
figure;
plot(D/n,1e3*z(:,1));
ylim([-1,0]); grid on;
title("Damage parameter for distributed impacts"); xlabel("Damage parameter [-]"); ylabel("z [mm]");

% Calculate corresponding lifetime values
volume = 4/3*pi*(1e-3)^3;               % Droplet volume [m^3]
area = pi*(r_max)^2;                    % Considered area [m^2]
specific_impacts = (n/max(D))/area;     % Specific impacts [#/m2]
impingement = specific_impacts*volume;  % Impingement [m]

%% Summarize results
results = table(lifetime_single_point,specific_impacts,impingement)